



class LigandSizeExceed(Exception):
    pass

class MissResidueFlag(Exception):
    pass

class SameMolecule(Exception):
    pass

class ExecutableNotInPath(Exception):
    pass

class OldPythonVersion(Exception):
    pass

class LigandPreparationError(Exception):
    pass

class NoGeometryAroundMetal(Exception):
    pass

class EnvVariableNotFound(Exception):
    pass

class WrongAtomSpecified(Exception):
    pass

class NotCenterOfWaterBox(Exception):
    pass

class OutInError(Exception):
    pass

class TemplateFileNotFound(Exception):
    pass

class RotamersFileNotFound(Exception):
    pass

class WrongAtomStringFormat(Exception):
    pass

class WrongYamlFile(Exception):
    pass
